M100FALSE - an interpreter in TRS-80 Model 100 BASIC
by Chris McCallum (vyv@sympatico.ca)  2013

Synopsis
	M100False is an interpreter for the FALSE language, a small, stack based, postfix, programming language.  It has been written in Model 100 BASIC.  It is a submission to the 2013 Club100 programming contest.

What is FALSE?
	FALSE is a language designed by Wouter van Oortmerssen in 1993 and implemented originally on an Amiga computer.  The main goal was to have a simple and small language to allow for small compilers.  FALSE uses a stack and reverse polish notation in ways similar to FORTH.  Commands in the language are single characters.  It is commonly refered to as an Esoteric Programming Language.

Usage
	When run, M100FALSE will ask for the name of a FALSE source file to load and execute.  Following a successful execution, the interpreter will display the running time of the file and leave the user at the BASIC prompt.  The user can rerun the program or exit to the MENU.  If an error is detected during the run, the interpreter will indicate the type of error, print some information, and leave the user at the BASIC prompt.

Goals
	To win a programming contest. :)
	To have a compatible FALSE interpreter.
	To have the interpreter run quickly.
	To use interesting BASIC programing ideas.

The Language
	Spaces are only required to separate numbers.  Numbers are signed 16bit values.  There are 26 signed 16bit variables, the lowercase letters.  A variable can hold a number or an address to a lambda routine.  Lambda routines are surrounded by [ and ].  Lambdas, numbers, and variables all push a value onto the stack.  The commands of the language operate by popping values from the stack, performing their function on the values, then pushing a result onto the stack.  Commands include arithmetic, stack manipulation, comparison, if, while, and simple I/O.

A few simple examples
	Add two numbers and show sum:  2 2+.
	Solve 2*(4+(9/3))           :  9 3/4+2*
                                 or :  2 4 9 3/+*
	Store value in variable     :  123a:
	Retrieve and show value     :  a;.
	Print Hello World!          :  "Hello World!"
	Store lambda in variable,
	retieve and execute lambda  :  [2 2+.]a:a;!

The BASIC code
	I have tried to keep the BASIC code as simple as possible so that others can read and understand it easily.  The main interpreter loop runs in a FOR loop using the variable I.  It takes the next character in the loaded file and branches to an appropriate routine.  These routines are collected together in a sort of jumptable in the lowest numbered lines.  Rather than RETURNing, the routines are GOTO'd.  A NEXT at the end of the routine sends it back to the interpreter loop for the next character.  It is therefore important to keep this in mind when adding routines.  In my benchmarking this one optimization reduced the runtime of PRIME(10) from 50 seconds to 46 seconds.
	Throughout the program I tried to keep the number of GOTO's and GOSUB's to a minimum.  In most places they were replaced with a FOR/NEXT loop with the exit value manipulated according to whether or not it is time to exit. 	The error handler uses the built in ON ERROR.  This routine tries to determine the type of error (in BASIC or in FALSE) using assumptions about the ERR, ERL, and value of variables.  It is not perfect, nor is it heavily tested, but should give some help to the desperate FALSE programmer.

Notes
	There are some differences between the original FALSE language and the M100FALSE language.  There are no commands for flushing buffers or for inserting machine code.  Stack cell size is 16 bits instead of 32 bits.  The pick command is P instead of the original Amiga character.
	If you don't care about the runtime, or don't want M100FALSE changing your clock, you can remove the references to TIME$ from the source code.
	Several ASCII characters are not printed on the keyboard of the Model 100.  Not (~) is on graph-], or (|) is on graph-_, swap (\) is on graph--, and comments ({}) are on graph-9 and graph-0.
	Currently, it is not an error when the interpreter reaches a character it does not recognize.  In this case it simply ignores the character and moves on.
	It may be necessary to increase the string space using a CLEAR statement.  The sample programs don't need this done to work, but any longer programs that cause an out of string space error will.  Also, long pauses in the execution of a FALSE file may indicate garbage collection, in which case increasing the string space may be the best way to keep the collections to a minimum.
	The store (:) and retrieve (;) commands work with actual memory addresses.  This way the equivalent of 16bit peeks and pokes can be done anywhere in memory.  This is also a great way to completely mess up the M100 requiring a cold restart.  Be careful!

The Hardest Part
	Writing documentation :)  As far as the source code goes, the tricky relationship between the commands that use the return stack (namely ] and # in particular) was the hardest thing to wrap my head around.

Further developments
	Having the program take input from the keyboard if no file name is given would be nice and should be doable.
	There are surely more ways to optimize the interpreter.
	The error handler could use some improvement.
	It would be really neat to have the apply (!) command work with real memory addresses.  Then the built in M100 assembly routines could be tapped into from a FALSE program.
	Model 100 specific commands could (and should) be added.  In particular, file access, and use of the NADSbox.

Language Reference
	+ - * / add, subtract, multiply, divide
	$ dup
	% drop
	\ swap
	@ rotate
	P pick
	_ negate
	= equal
	> greater than
	& and
	| or
	~ not
	: store
	; retrieve
	[ ] lambda
	! apply
	? if
	# while
	' push char
	^ get char
	. print number
	, print char
	" print line
	{ } comment

Online References
	strlen.com/false-language - Wouters FALSE webpage
	esolangs.org/wiki/Category:FALSE
	rosettacode.org/wiki/Category:FALSE - Lot's of good example FALSE programs
	en.wikipedia.org/wiki/FALSE

Outro
	It should be obvious that this doc file is not a tutorial.  Using the online references it should be fairly easy to learn this simple language, in particular for those familiar with FORTH type languages.

